//
// Created by doggo on 04/02/2021.
//

#include "../headers/lex.hpp"
#include <iostream>
using namespace std;

vector<tok> resVal;

void separate() {resVal.emplace_back(tok::SEPARATOR, "");}

string generatedStr;
bool isLexingString = false;

tok::type getIdentifierCharType(const char& Char) {
    if (isalpha(Char)) return tok::IDENTIFIER;
    else if (isalnum(Char)) return tok::NUMBER;
    else return tok::UNEXPECTED;
}

vector<tok>& lex(const string& in)
{
    for (const char& current : in) {
        if (!isLexingString) {
            if (!generatedStr.empty()) {
                resVal.emplace_back(tok::STRING, generatedStr);
                generatedStr = "";
            }
            switch (current) {
                case '#': resVal.emplace_back(tok::TAG, "#"); break;
                case '=': resVal.emplace_back(tok::DEFINE, "="); break;
                case '(': resVal.emplace_back(tok::LPAR, "("); break;
                case ')': resVal.emplace_back(tok::RPAR, ")"); break;
                case '{': resVal.emplace_back(tok::LBRACE, "{"); break;
                case '}': resVal.emplace_back(tok::RBRACE, "}"); break;
                case '[': resVal.emplace_back(tok::LBRACKET, "["); break;
                case ']': resVal.emplace_back(tok::RBRACKET, "]"); break;
                case '+': resVal.emplace_back(tok::PLUS, "+"); break;
                case '-': resVal.emplace_back(tok::HYPHEN, "-"); break;
                case '>': resVal.emplace_back(tok::LCOMP, ">"); break;
                case '<': resVal.emplace_back(tok::RCOMP, "<"); break;
                case '\'': resVal.emplace_back(tok::SQUOTE, "\'"); break;
                case '"': isLexingString = true; break;
                case ' ': case '\t': case '\r':
                case '\n': separate(); break;
                default: { // TODO Lex entire identifiers and numbers
                    tok::type type = getIdentifierCharType(current);
                    if (type != tok::UNEXPECTED) resVal.emplace_back(type, string(1, current));
                }
            }
        } else {
            if ((current == '"')) { // TODO Permit the usage of double quotes in strings using '\' (an example would be "\"")
                isLexingString = false;
            } else generatedStr.append(string(1, current));
        }
    }
    /* Test
    for (const auto& it : resVal) {
        if (it.toktype == tok::STRING) cout << '"' << it.toktext << '"' << '\n';
        if (it.toktype == tok::NUMBER) cout << it.toktext << "= number\n";
        if (it.toktype == tok::IDENTIFIER) cout << it.toktext << '\n';
    }
    */
    return resVal;
}